@@ -421,8 +421,8 @@ start_server {tags {"multi"}} {
         r exec
 
         assert_replication_stream $repl {
-            {select *}
             {multi}
+            {select *}
             {set foo{t} bar}
             {set foo2{t} bar2}
             {set foo3{t} bar3}
@@ -446,8 +446,8 @@ start_server {tags {"multi"}} {
         r exec
 
         assert_replication_stream $repl {
-            {select *}
             {multi}
+            {select *}
             {set foo{t} bar}
             {select *}
             {set foo2{t} bar2}
@@ -891,14 +891,14 @@ start_server {tags {"multi"}} {
         set res [r read]
         assert_equal $res "+OK"
         set res [r read]
-        r readraw 1
+        r readraw 0
         set _ $res
     } {*CONFIG SET failed*}
     
     test "Flushall while watching several keys by one client" {
         r flushall
-        r mset a a b b
-        r watch b a
+        r mset a{t} a b{t} b
+        r watch b{t} a{t}
         r flushall
         r ping
      }
@@ -912,8 +912,8 @@ start_server {overrides {appendonly {yes} appendfilename {appendonly.aof} append
         r flushall
         r exec
         assert_aof_content $aof {
-            {select *}
             {multi}
+            {select *}
             {set *}
             {flushall}
             {exec}
